@@ -105,6 +105,7 @@ void set_timeout(int timeleft)
 static RETSIGTYPE timeout_handler (int signal)
 /* handle SIGALRM signal indicating a server timeout */
 {
+    (void)signal;
     if(stage != STAGE_IDLE) {
 	timeoutcount++;
 	longjmp(restart, THROW_TIMEOUT);
@@ -115,6 +116,7 @@ static RETSIGTYPE timeout_handler (int signal)
 static RETSIGTYPE sigpipe_handler (int signal)
 /* handle SIGPIPE signal indicating a broken stream socket */
 {
+    (void)signal;
     longjmp(restart, THROW_SIGPIPE);
 }
 
@@ -140,7 +142,6 @@ int socket;		/* socket to server host */
 char *canonical;	/* server name */
 char *principal;
 {
-    char * host_primary;
     KTEXT ticket;
     MSG_DAT msg_data;
     CREDENTIALS cred;
@@ -214,19 +215,19 @@ const char *canonical;  /* server name */
     krb5_init_context(&context);
     krb5_auth_con_init(context, &auth_context);
 
-    if (retval = krb5_cc_default(context, &ccdef)) {
+    if ((retval = krb5_cc_default(context, &ccdef))) {
         report(stderr, "krb5_cc_default: %s\n", error_message(retval));
         return(PS_ERROR);
     }
 
-    if (retval = krb5_cc_get_principal(context, ccdef, &client)) {
+    if ((retval = krb5_cc_get_principal(context, ccdef, &client))) {
         report(stderr, "krb5_cc_get_principal: %s\n", error_message(retval));
         return(PS_ERROR);
     }
 
-    if (retval = krb5_sname_to_principal(context, canonical, "pop",
+    if ((retval = krb5_sname_to_principal(context, canonical, "pop",
            KRB5_NT_UNKNOWN,
-           &server)) {
+           &server))) {
         report(stderr, "krb5_sname_to_principal: %s\n", error_message(retval));
         return(PS_ERROR);
     }
@@ -331,7 +332,7 @@ static void send_size_warnings(struct query *ctl)
      * but it's not a disaster, either, since the skipped mail will not
      * be deleted.
      */
-    if (open_warning_by_mail(ctl, (struct msgblk *)NULL))
+    if (open_warning_by_mail(ctl))
 	return;
     stuff_warning(iana_charset, ctl,
 	   GT_("Subject: Fetchmail oversized-messages warning"));
@@ -358,7 +359,7 @@ static void send_size_warnings(struct query *ctl)
 	if (current->val.status.num == 0 && current->val.status.mark)
 	{
 	    nbr = current->val.status.mark;
-	    size = atoi((const char *)current->id);
+	    size = atoi(current->id);
 	    if (ctl->limitflush)
 		stuff_warning(NULL, ctl,
 			GT_("  %d msg %d octets long deleted by fetchmail."),
@@ -380,7 +381,7 @@ static void send_size_warnings(struct query *ctl)
     close_warning_by_mail(ctl, (struct msgblk *)NULL);
 }
 
-static void mark_oversized(struct query *ctl, int num, int size)
+static void mark_oversized(struct query *ctl, int size)
 /* mark a message oversized */
 {
     struct idlist *current=NULL, *tmp=NULL;
@@ -442,7 +443,7 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 
 	/* Time to allocate memory to store the sizes */
 	xfree(*msgsizes);
-	*msgsizes = xmalloc(sizeof(int) * fetchsizelimit);
+	*msgsizes = (int *)xmalloc(sizeof(int) * fetchsizelimit);
     }
 
     /*
@@ -546,7 +547,7 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 	{
 	    if (msgcode == MSGLEN_TOOLARGE)
 	    {
-		mark_oversized(ctl, num, msgsize);
+		mark_oversized(ctl, msgsize);
 		if (!ctl->limitflush)
 		    suppress_delete = TRUE;
 	    }
@@ -641,13 +642,13 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 	    /* tell server we got it OK and resynchronize */
 	    if (separatefetchbody && ctl->server.base_protocol->trail)
 	    {
-		if (outlevel >= O_VERBOSE && !is_a_file(1))
+		if (outlevel >= O_VERBOSE && !is_a_file(1) && !run.use_syslog)
 		{
 		    fputc('\n', stdout);
 		    fflush(stdout);
 		}
 
-		if ((err = (ctl->server.base_protocol->trail)(mailserver_socket, ctl, num, tag)))
+		if ((err = (ctl->server.base_protocol->trail)(mailserver_socket, ctl, tag)))
 		    return(err);
 	    }
 
@@ -699,13 +700,13 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 		/* tell server we got it OK and resynchronize */
 		if (ctl->server.base_protocol->trail)
 		{
-		    if (outlevel >= O_VERBOSE && !is_a_file(1))
+		    if (outlevel >= O_VERBOSE && !is_a_file(1) && !run.use_syslog)
 		    {
 			fputc('\n', stdout);
 			fflush(stdout);
 		    }
 
-		    err = (ctl->server.base_protocol->trail)(mailserver_socket, ctl, num, tag);
+		    err = (ctl->server.base_protocol->trail)(mailserver_socket, ctl, tag);
 		    if (err != 0)
 			return(err);
 		}
@@ -774,7 +775,7 @@ flagthemail:
 	    if (outlevel > O_SILENT) 
 		report(stdout, GT_(" retained\n"));
 	}
-	else if (ctl->server.base_protocol->delete
+	else if (ctl->server.base_protocol->delete_msg
 		 && !suppress_delete
 		 && ((msgcode >= 0 && !ctl->keep)
 		     || (msgcode == MSGLEN_OLD && ctl->flush)
@@ -783,7 +784,7 @@ flagthemail:
 	    (*deletions)++;
 	    if (outlevel > O_SILENT) 
 		report_complete(stdout, GT_(" flushed\n"));
-	    err = (ctl->server.base_protocol->delete)(mailserver_socket, ctl, num);
+	    err = (ctl->server.base_protocol->delete_msg)(mailserver_socket, ctl, num);
 	    if (err != 0)
 		return(err);
 	}
@@ -913,7 +914,7 @@ static int do_session(
 	     * timeouts just mean the frequency of mail is low.
 	     */
 	    if (timeoutcount > MAX_TIMEOUTS 
-		&& !open_warning_by_mail(ctl, (struct msgblk *)NULL))
+		&& !open_warning_by_mail(ctl))
 	    {
 		stuff_warning(iana_charset, ctl,
 			      GT_("Subject: fetchmail sees repeated timeouts"));
@@ -940,7 +941,7 @@ static int do_session(
     {
 	/* setjmp returned zero -> normal operation */
 	char buf[MSGBUFSIZE+1], *realhost;
-	int count, new, bytes;
+	int count, newm, bytes;
 	int fetches, dispatches, oldphase;
 	struct idlist *idp;
 
@@ -1027,8 +1028,16 @@ static int do_session(
 		else
 		{
 		    xfree(ctl->server.truename);
-		    ctl->server.truename = xstrdup(res->ai_canonname);
-		    ctl->server.trueaddr = xmalloc(res->ai_addrlen);
+		    /* Older FreeBSD versions return NULL in ai_canonname
+		     * if they cannot canonicalize, rather than copying
+		     * the queryname here, as IEEE Std 1003.1-2001
+		     * requires. Work around NULL. */
+		    if (res->ai_canonname != NULL) {
+			ctl->server.truename = xstrdup(res->ai_canonname);
+		    } else {
+			ctl->server.truename = xstrdup(ctl->server.queryname);
+		    }
+		    ctl->server.trueaddr = (struct sockaddr *)xmalloc(res->ai_addrlen);
 		    ctl->server.trueaddr_len = res->ai_addrlen;
 		    memcpy(ctl->server.trueaddr, res->ai_addr, res->ai_addrlen);
 		    freeaddrinfo(res);
@@ -1068,7 +1077,7 @@ static int do_session(
 		 * where your SLIP or PPP link is down...
 		 */
 		/* warn the system administrator */
-		if (open_warning_by_mail(ctl, (struct msgblk *)NULL) == 0)
+		if (open_warning_by_mail(ctl) == 0)
 		{
 		    stuff_warning(iana_charset, ctl,
 			 GT_("Subject: Fetchmail unreachable-server warning."));
@@ -1181,7 +1190,7 @@ static int do_session(
 			&& !ctl->wehavesentauthnote
 			&& ((ctl->wehaveauthed && ++ctl->authfailcount >= 10)
 			    || (!ctl->wehaveauthed && ++ctl->authfailcount >= 3))
-			&& !open_warning_by_mail(ctl, (struct msgblk *)NULL))
+			&& !open_warning_by_mail(ctl))
 		    {
 			ctl->wehavesentauthnote = 1;
 			stuff_warning(iana_charset, ctl,
@@ -1192,13 +1201,14 @@ static int do_session(
 				      GT_("Fetchmail could not get mail from %s@%s.\n"), 
 				      ctl->remotename,
 				      ctl->server.truename);
-			if (ctl->wehaveauthed)
+			if (ctl->wehaveauthed) {
 			    stuff_warning(NULL, ctl, GT_("\
 The attempt to get authorization failed.\n\
 Since we have already succeeded in getting authorization for this\n\
 connection, this is probably another failure mode (such as busy server)\n\
 that fetchmail cannot distinguish because the server didn't send a useful\n\
-error message.\n\
+error message."));
+			    stuff_warning(NULL, ctl, GT_("\
 \n\
 However, if you HAVE changed your account details since starting the\n\
 fetchmail daemon, you need to stop the daemon, change your configuration\n\
@@ -1207,7 +1217,7 @@ of fetchmail, and then restart the daemon.\n\
 The fetchmail daemon will continue running and attempt to connect\n\
 at each cycle.  No future notifications will be sent until service\n\
 is restored."));
-			else
+			} else {
 			    stuff_warning(NULL, ctl, GT_("\
 The attempt to get authorization failed.\n\
 This probably means your password is invalid, but some servers have\n\
@@ -1217,6 +1227,7 @@ because they don't send useful error messages on login failure.\n\
 The fetchmail daemon will continue running and attempt to connect\n\
 at each cycle.  No future notifications will be sent until service\n\
 is restored."));
+			}
 			close_warning_by_mail(ctl, (struct msgblk *)NULL);
 		    }
 		}
@@ -1255,7 +1266,7 @@ is restored."));
 			   GT_("Authorization OK on %s@%s\n"),
 			   ctl->remotename,
 			   ctl->server.truename);
-		    if (!open_warning_by_mail(ctl, (struct msgblk *)NULL))
+		    if (!open_warning_by_mail(ctl))
 		    {
 			stuff_warning(iana_charset, ctl,
 			      GT_("Subject: fetchmail authentication OK on %s@%s"), 
@@ -1304,7 +1315,7 @@ is restored."));
 
 		/* compute # of messages and number of new messages waiting */
 		stage = STAGE_GETRANGE;
-		err = (ctl->server.base_protocol->getrange)(mailserver_socket, ctl, (const char *)idp->id, &count, &new, &bytes);
+		err = (ctl->server.base_protocol->getrange)(mailserver_socket, ctl, idp->id, &count, &newm, &bytes);
 		if (err != 0)
 		    goto cleanUp;
 
@@ -1322,11 +1333,11 @@ is restored."));
 			report(stdout, GT_("Polling %s\n"), ctl->server.truename);
 		    else if (count != 0)
 		    {
-			if (new != -1 && (count - new) > 0)
+			if (newm != -1 && (count - newm) > 0)
 			    report_build(stdout, ngettext("%d message (%d %s) for %s", "%d messages (%d %s) for %s", (unsigned long)count),
 				  count,
-				  count-new, 
-				  ngettext("seen", "seen", (unsigned long)count-new),
+				  count - newm, 
+				  ngettext("seen", "seen", (unsigned long)count-newm),
 				  buf);
 			else
 			    report_build(stdout, ngettext("%d message for %s",
@@ -1352,9 +1363,9 @@ is restored."));
 
 		if (check_only)
 		{
-		    if (new == -1 || ctl->fetchall)
-			new = count;
-		    fetches = new;	/* set error status correctly */
+		    if (newm == -1 || ctl->fetchall)
+			newm = count;
+		    fetches = newm;	/* set error status correctly */
 		    /*
 		     * There used to be a `goto noerror' here, but this
 		     * prevented checking of multiple folders.  This
@@ -1373,7 +1384,7 @@ is restored."));
 		     * count, and allocate a malloc area that would overlap
 		     * a portion of the stack.
 		     */
-		    if (count > INT_MAX/sizeof(int))
+		    if ((unsigned)count > INT_MAX/sizeof(int))
 		    {
 			report(stderr, GT_("bogus message count!"));
 			err = PS_PROTOCOL;
@@ -1394,7 +1405,7 @@ is restored."));
 			!(proto->getpartialsizes && NUM_NONZERO(ctl->fetchsizelimit)))
 		    {
 			xfree(msgsizes);
-			msgsizes = xmalloc(sizeof(int) * count);
+			msgsizes = (int *)xmalloc(sizeof(int) * count);
 			for (i = 0; i < count; i++)
 			    msgsizes[i] = 0;
 
@@ -1572,10 +1583,9 @@ closeUp:
     return(err);
 }
 
-int do_protocol(ctl, proto)
-/* retrieve messages from server using given protocol method table */
-struct query *ctl;		/* parsed options with merged-in defaults */
-const struct method *proto;	/* protocol method table */
+/** retrieve messages from server using given protocol method table */
+int do_protocol(struct query *ctl /** parsed options with merged-in defaults */,
+		const struct method *proto /** protocol method table */)
 {
     int	err;
 
