@@ -130,8 +130,8 @@ static RETSIGTYPE donothing(int sig)
 static void printcopyright(FILE *fp) {
 	fprintf(fp, GT_("Copyright (C) 2002, 2003 Eric S. Raymond\n"
 		   "Copyright (C) 2004 Matthias Andree, Eric S. Raymond, Rob F. Funk, Graham Wilson\n"
-		   "Copyright (C) 2005 Matthias Andree, Sunil Shetye\n"
-		   "Copyright (C) 2006 Matthias Andree\n"));
+		   "Copyright (C) 2005-2006 Matthias Andree, Sunil Shetye\n"
+		   ));
 	fprintf(fp, GT_("Fetchmail comes with ABSOLUTELY NO WARRANTY. This is free software, and you\n"
 		   "are welcome to redistribute it under certain conditions. For details,\n"
 		   "please see the file COPYING in the source or documentation directory.\n"));
@@ -302,6 +302,18 @@ int main(int argc, char **argv)
 #endif
 	report_init((run.poll_interval == 0 || nodetach) && !run.logfile);
 
+#ifdef POP3_ENABLE
+    /* initialize UID handling */
+    {
+	int st;
+
+	if (!versioninfo && (st = prc_filecheck(run.idfile, !versioninfo)) != 0)
+	    exit(st);
+	else
+	    initialize_saved_lists(querylist, run.idfile);
+    }
+#endif /* POP3_ENABLE */
+
     /* construct the lockfile */
     lock_setup();
 
@@ -534,7 +546,7 @@ int main(int argc, char **argv)
     if (run.poll_interval)
     {
 	if (!nodetach)
-	    daemonize(run.logfile, terminate_run);
+	    daemonize(run.logfile);
 	report(stdout, GT_("starting fetchmail %s daemon \n"), VERSION);
 
 	/*
@@ -958,12 +970,12 @@ static int load_params(int argc, char **argv, int optind)
     def_opts.remotename = user;
     def_opts.listener = SMTP_MODE;
     def_opts.fetchsizelimit = 100;
-    def_opts.fastuidl = 10;
+    def_opts.fastuidl = 4;
 
     /* get the location of rcfile */
     rcfiledir[0] = 0;
     p = strrchr (rcfile, '/');
-    if (p && (p - rcfile) < sizeof (rcfiledir)) {
+    if (p && (size_t)(p - rcfile) < sizeof (rcfiledir)) {
 	*p = 0;			/* replace '/' by '0' */
 	strlcpy (rcfiledir, rcfile, sizeof(rcfiledir));
 	*p = '/';		/* restore '/' */
@@ -1242,9 +1254,9 @@ static int load_params(int argc, char **argv, int optind)
 		{
 		    char	*cp;
 
-		    if (!(cp = strrchr((char *)idp->id, '/')) ||
-			    ++cp, (0 == strcmp(cp, SMTP_PORT))
-				|| servport(cp) == SMTP_PORT_NUM)
+		    if (!(cp = strrchr(idp->id, '/'))
+			|| (0 == strcmp(cp + 1, SMTP_PORT))
+			|| servport(cp + 1) == SMTP_PORT_NUM)
 		    {
 			(void) fprintf(stderr,
 				       GT_("%s configuration invalid, LMTP can't use default SMTP port\n"),
@@ -1266,14 +1278,6 @@ static int load_params(int argc, char **argv, int optind)
 	}
     }
 
-#ifdef POP3_ENABLE
-    /* initialize UID handling */
-    if (!versioninfo && (st = prc_filecheck(run.idfile, !versioninfo)) != 0)
-	exit(st);
-    else
-	initialize_saved_lists(querylist, run.idfile);
-#endif /* POP3_ENABLE */
-
     /*
      * If the user didn't set a last-resort user to get misaddressed
      * multidrop mail, set an appropriate default here.
@@ -1358,7 +1362,8 @@ static const int autoprobe[] =
 static int query_host(struct query *ctl)
 /* perform fetch transaction with single host */
 {
-    int i, st = 0;
+    size_t i;
+    int st = 0;
 
     /*
      * If we're syslogging the progress messages are automatically timestamped.
@@ -1603,7 +1608,7 @@ static void dump_params (struct runctl *runp,
 
 		printf(GT_("  Selected mailboxes are:"));
 		for (idp = ctl->mailboxes; idp; idp = idp->next)
-		    printf(" %s", (char *)idp->id);
+		    printf(" %s", idp->id);
 		printf("\n");
 	    }
 	    printf(ctl->fetchall
@@ -1693,7 +1698,7 @@ static void dump_params (struct runctl *runp,
 	    printf(GT_("  Domains for which mail will be fetched are:"));
 	    for (idp = ctl->domainlist; idp; idp = idp->next)
 	    {
-		printf(" %s", (char *)idp->id);
+		printf(" %s", idp->id);
 		if (!idp->val.status.mark)
 		    printf(GT_(" (default)"));
 	    }
@@ -1713,7 +1718,7 @@ static void dump_params (struct runctl *runp,
 		       ctl->listener);
 		for (idp = ctl->smtphunt; idp; idp = idp->next)
 		{
-		    printf(" %s", (char *)idp->id);
+		    printf(" %s", idp->id);
 		    if (!idp->val.status.mark)
 			printf(GT_(" (default)"));
 		}
@@ -1771,9 +1776,9 @@ static void dump_params (struct runctl *runp,
 		    {
 			for (idp = ctl->localnames; idp; idp = idp->next)
 			    if (idp->val.id2)
-				printf("\t%s -> %s\n", (char *)idp->id, (char *)idp->val.id2);
+				printf("\t%s -> %s\n", idp->id, idp->val.id2);
 			    else
-				printf("\t%s\n", (char *)idp->id);
+				printf("\t%s\n", idp->id);
 			if (ctl->wildcard)
 			    fputs("\t*\n", stdout);
 		    }
@@ -1812,7 +1817,7 @@ static void dump_params (struct runctl *runp,
 
 			    printf(GT_("  Predeclared mailserver aliases:"));
 			    for (idp = ctl->server.akalist; idp; idp = idp->next)
-				printf(" %s", (char *)idp->id);
+				printf(" %s", idp->id);
 			    putchar('\n');
 			}
 			if (ctl->server.localdomains)
@@ -1821,7 +1826,7 @@ static void dump_params (struct runctl *runp,
 
 			    printf(GT_("  Local domains:"));
 			    for (idp = ctl->server.localdomains; idp; idp = idp->next)
-				printf(" %s", (char *)idp->id);
+				printf(" %s", idp->id);
 			    putchar('\n');
 			}
 		    }
@@ -1862,7 +1867,7 @@ static void dump_params (struct runctl *runp,
 		printf(GT_("  %d UIDs saved.\n"), count);
 		if (outlevel >= O_VERBOSE)
 		    for (idp = ctl->oldsaved; idp; idp = idp->next)
-			printf("\t%s\n", (char *)idp->id);
+			printf("\t%s\n", idp->id);
 	    }
 	}
 
