@@ -49,6 +49,7 @@ enum {
     LA_FETCHSIZELIMIT,
     LA_FASTUIDL,
     LA_LIMITFLUSH,
+    LA_IDLE
 };
 
 /* options still left: CgGhHjJoORTWxXYz */
@@ -78,6 +79,7 @@ static const struct option longoptions[] = {
   {"protocol",	required_argument, (int *) 0, 'p' },
   {"proto",	required_argument, (int *) 0, 'p' },
   {"uidl",	no_argument,	   (int *) 0, 'U' },
+  {"idle",	no_argument,	   (int *) 0, LA_IDLE},
   {"port",	required_argument, (int *) 0, 'P' },
   {"service",	required_argument, (int *) 0, 'P' },
   {"auth",	required_argument, (int *) 0, LA_AUTH},
@@ -89,6 +91,7 @@ static const struct option longoptions[] = {
   {"username",	required_argument, (int *) 0, 'u' },
 
   {"all",	no_argument,	   (int *) 0, 'a' },
+  {"fetchall",	no_argument,	   (int *) 0, 'a' },
   {"nokeep",	no_argument,	   (int *) 0, 'K' },
   {"keep",	no_argument,	   (int *) 0, 'k' },
   {"flush",	no_argument,	   (int *) 0, 'F' },
@@ -212,12 +215,11 @@ static int xatoi(char *s, int *errflagptr)
 #endif
 }
 
-int parsecmdline (argc, argv, rctl, ctl)
-/* parse and validate the command line options */
-int argc;		/* argument count */
-char **argv;		/* argument strings */
-struct runctl *rctl;	/* global run controls to modify */
-struct query *ctl;	/* option record to be initialized */
+/** parse and validate the command line options */
+int parsecmdline (int argc /** argument count */,
+		  char **argv /** argument strings */,
+		  struct runctl *rctl /** global run controls to modify */,
+		  struct query *ctl /** option record to initialize */)
 {
     /*
      * return value: if positive, argv index of last parsed option + 1
@@ -334,6 +336,9 @@ struct query *ctl;	/* option record to be initialized */
 	case 'U':
 	    ctl->server.uidl = FLAG_TRUE;
 	    break;
+	case LA_IDLE:
+	    ctl->idle = FLAG_TRUE;
+	    break;
 	case 'P':
 	    ctl->server.service = optarg;
 	    break;
