@@ -300,7 +300,7 @@ static int send_bouncemail(struct query *ctl, struct msgblk *msg,
 
     /* our first duty is to keep the sacred foo counters turning... */
     snprintf(boundary, sizeof(boundary), "foo-mani-padme-hum-%ld-%ld-%ld", 
-	    (long)getpid(), (long)getppid(), time(NULL));
+	    (long)getpid(), (long)getppid(), (long)time(NULL));
 
     if (outlevel >= O_VERBOSE)
 	report(stdout, GT_("SMTP: (bounce-message body)\n"));
@@ -920,7 +920,7 @@ static int open_smtp_sink(struct query *ctl, struct msgblk *msg,
     for (idp = msg->recipients; idp; idp = idp->next)
 	total_addresses++;
 #ifdef EXPLICIT_BOUNCE_ON_BAD_ADDRESS
-    from_responses = xmalloc(sizeof(char *) * total_addresses);
+    from_responses = (char **)xmalloc(sizeof(char *) * total_addresses);
 #endif /* EXPLICIT_BOUNCE_ON_BAD_ADDRESS */
     for (idp = msg->recipients; idp; idp = idp->next)
 	if (idp->val.status.mark == XMIT_ACCEPT)
@@ -1070,6 +1070,7 @@ static int open_mda_sink(struct query *ctl, struct msgblk *msg,
     int	length = 0, fromlen = 0, nameslen = 0;
     char	*names = NULL, *before, *after, *from = NULL;
 
+    (void)bad_addresses;
     xfree(ctl->destaddr);
     ctl->destaddr = xstrdup("localhost");
 
@@ -1425,7 +1426,7 @@ int close_sink(struct query *ctl, struct msgblk *msg, flag forward)
 		char	**responses;
 
 		/* eat the RFC2033-required responses, saving errors */ 
-		responses = xmalloc(sizeof(char *) * lmtp_responses);
+		responses = (char **)xmalloc(sizeof(char *) * lmtp_responses);
 		for (errors = i = 0; i < lmtp_responses; i++)
 		{
 		    if ((smtp_err = SMTP_ok(ctl->smtp_socket, ctl->smtphostmode))
@@ -1468,7 +1469,7 @@ unrecov:
     return(TRUE);
 }
 
-int open_warning_by_mail(struct query *ctl, struct msgblk *msg)
+int open_warning_by_mail(struct query *ctl)
 /* set up output sink for a mailed warning to calling user */
 {
     int	good, bad;
